/*
 * EP93xx Power Management Routines
 *
 * Copyright (c) 2005 Atmark Techno, Inc.
 */

#include <linux/config.h>
#include <linux/init.h>
#include <linux/pm.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/sysctl.h>
#include <linux/errno.h>

#include <asm/hardware.h>
#include <asm/memory.h>
#include <asm/system.h>
#include <asm/io.h>
#include <asm/arch/regmap.h>
#include <asm/arch/regs_syscon.h>

#define SAVE(x)		sleep_save[SLEEP_SAVE_##x] = inl(x)
#define LOAD(x)		sleep_save[SLEEP_SAVE_##x]
#define RESTORE(x)	outl(sleep_save[SLEEP_SAVE_##x], x)

enum {	SLEEP_SAVE_START = 0,

#ifndef CONFIG_PM_UNUSE_SLEEP_SAVE
	SLEEP_SAVE_SYSCON_CLKSET2,
	SLEEP_SAVE_SYSCON_VIDDIV, SLEEP_SAVE_SYSCON_MIRDIV,
	SLEEP_SAVE_SYSCON_I2SDIV,
	SLEEP_SAVE_SYSCON_CLKSET1,
#endif
	SLEEP_SAVE_SYSCON_DEVCFG,
#ifndef CONFIG_PM_UNUSE_SLEEP_SAVE
	SLEEP_SAVE_GPIO_AINTEN, SLEEP_SAVE_GPIO_BINTEN,
	SLEEP_SAVE_GPIO_FINTEN,
	SLEEP_SAVE_GPIO_PADDR, SLEEP_SAVE_GPIO_PBDDR,
	SLEEP_SAVE_GPIO_PCDDR, SLEEP_SAVE_GPIO_PDDDR,
	SLEEP_SAVE_GPIO_PEDDR,
#ifdef CONFIG_PM_UNUSE_PCMCIA
	SLEEP_SAVE_GPIO_PFDDR,
#endif
	SLEEP_SAVE_GPIO_PGDDR, SLEEP_SAVE_GPIO_PHDDR,
	SLEEP_SAVE_GPIO_PADR, SLEEP_SAVE_GPIO_PBDR,
	SLEEP_SAVE_GPIO_PCDR, SLEEP_SAVE_GPIO_PDDR,
	SLEEP_SAVE_GPIO_PEDR,
#ifdef CONFIG_PM_UNUSE_PCMCIA
	SLEEP_SAVE_GPIO_PFDR,
#endif
	SLEEP_SAVE_GPIO_PGDR, SLEEP_SAVE_GPIO_PHDR,
#endif

	SLEEP_SAVE_SIZE
};

static inline void Unlock_SYSLCK(void)
{
	outl(0xaa, SYSCON_SWLOCK);
}

int pm_do_suspend(void)
{
	unsigned long ulTemp;
	unsigned long sleep_save[SLEEP_SAVE_SIZE];

	cli();

#ifndef CONFIG_PM_UNUSE_SLEEP_SAVE
	SAVE(SYSCON_CLKSET2);
	outl(LOAD(SYSCON_CLKSET2) & 0xfffbffff, SYSCON_CLKSET2);

	SAVE(SYSCON_VIDDIV);
	Unlock_SYSLCK();
	outl(0, SYSCON_VIDDIV);

	SAVE(SYSCON_MIRDIV);
	Unlock_SYSLCK();
	outl(0, SYSCON_MIRDIV);

	SAVE(SYSCON_I2SDIV);
	Unlock_SYSLCK();
	outl(0, SYSCON_I2SDIV);

	SAVE(SYSCON_CLKSET1);
	outl(0x097f98e7, SYSCON_CLKSET1);
#endif

	SAVE(SYSCON_DEVCFG);
#ifndef CONFIG_PM_UNUSE_SLEEP_SAVE
	Unlock_SYSLCK();
	outl((LOAD(SYSCON_DEVCFG) & ~0x00800000) | 0x09124d0e, SYSCON_DEVCFG);

	SAVE(GPIO_AINTEN);
	outl(0, GPIO_AINTEN);
	SAVE(GPIO_BINTEN);
	outl(0, GPIO_BINTEN);
	SAVE(GPIO_FINTEN);
#ifdef CONFIG_PM_UNUSE_PCMCIA
	outl(0, GPIO_FINTEN);
#else
	outl(0x40, GPIO_FINTEN);
#endif
	SAVE(GPIO_PADDR);
#if !defined(CONFIG_PM_UNUSE_PCMCIA) && defined(CONFIG_ARCH_ARMADILLO9)
	outl(LOAD(GPIO_PADDR) & 0x8, GPIO_PADDR);
#else
	outl(0, GPIO_PADDR);
#endif
	SAVE(GPIO_PBDDR);
	outl(0, GPIO_PBDDR);
	SAVE(GPIO_PCDDR);
	outl(0, GPIO_PCDDR);
	SAVE(GPIO_PDDDR);
	outl(0, GPIO_PDDDR);
	SAVE(GPIO_PEDDR);
	outl(0, GPIO_PEDDR);
#ifdef CONFIG_PM_UNUSE_PCMCIA
	SAVE(GPIO_PFDDR);
	outl(0, GPIO_PFDDR);
#endif
#if defined(CONFIG_PM_UNUSE_PCMCIA) || defined(CONFIG_ARCH_ARMADILLO9)
	outl(0, GPIO_PGDDR);
#else
	outl(LOAD(GPIO_PGDDR) & 0x7, GPIO_PGDDR);
#endif
	SAVE(GPIO_PHDDR);
	outl(0, GPIO_PHDDR);
	SAVE(GPIO_PADR);
#if !defined(CONFIG_PM_UNUSE_PCMCIA) && defined(CONFIG_ARCH_ARMADILLO9)
	outl(LOAD(GPIO_PADR) & 0x8, GPIO_PADR);
#else
	outl(0, GPIO_PADR);
#endif
	SAVE(GPIO_PBDR);
	outl(0, GPIO_PBDR);
	SAVE(GPIO_PCDR);
	outl(0, GPIO_PCDR);
	SAVE(GPIO_PDDR);
	outl(0, GPIO_PDDR);
	SAVE(GPIO_PEDR);
	outl(0, GPIO_PEDR);
#ifdef CONFIG_PM_UNUSE_PCMCIA
	SAVE(GPIO_PFDR);
	outl(0, GPIO_PFDR);
#endif
	SAVE(GPIO_PGDR);
#if defined(CONFIG_PM_UNUSE_PCMCIA) || defined(CONFIG_ARCH_ARMADILLO9)
	outl(0, GPIO_PGDR);
#else
	outl(LOAD(GPIO_PGDR) & 0x7, GPIO_PGDR);
#endif
	SAVE(GPIO_PHDR);
	outl(0, GPIO_PHDR);
#endif

	ulTemp = inl(SYSCON_DEVCFG) | SYSCON_DEVCFG_SHena;
	Unlock_SYSLCK();
	outl(ulTemp, SYSCON_DEVCFG);

	sti();

	/* go zzz */
	ulTemp = inl(SYSCON_STBY);

	cli();

	printk(KERN_DEBUG "*** made it back from resume\n");

#ifndef CONFIG_PM_UNUSE_SLEEP_SAVE
	RESTORE(GPIO_PHDR);
	RESTORE(GPIO_PGDR);
#ifdef CONFIG_PM_UNUSE_PCMCIA
	RESTORE(GPIO_PFDR);
#endif
	RESTORE(GPIO_PEDR);
	RESTORE(GPIO_PDDR);
	RESTORE(GPIO_PCDR);
	RESTORE(GPIO_PBDR);
	RESTORE(GPIO_PADR);
	RESTORE(GPIO_PHDDR);
	RESTORE(GPIO_PGDDR);
#ifdef CONFIG_PM_UNUSE_PCMCIA
	RESTORE(GPIO_PFDDR);
#endif
	RESTORE(GPIO_PEDDR);
	RESTORE(GPIO_PDDDR);
	RESTORE(GPIO_PCDDR);
	RESTORE(GPIO_PBDDR);
	RESTORE(GPIO_PADDR);
	RESTORE(GPIO_FINTEN);
	RESTORE(GPIO_BINTEN);
	RESTORE(GPIO_AINTEN);
#endif

	Unlock_SYSLCK();
	RESTORE(SYSCON_DEVCFG);

#ifndef CONFIG_PM_UNUSE_SLEEP_SAVE
	RESTORE(SYSCON_CLKSET1);

	Unlock_SYSLCK();
	RESTORE(SYSCON_I2SDIV);

	Unlock_SYSLCK();
	RESTORE(SYSCON_MIRDIV);

	Unlock_SYSLCK();
	RESTORE(SYSCON_VIDDIV);

	RESTORE(SYSCON_CLKSET2);
#endif

	sti();

	/*
	 * Restore the CPU frequency settings.
	 */
#ifdef CONFIG_CPU_FREQ
	cpufreq_restore();
#endif

	return 0;
}

unsigned long sleep_phys_sp(void *sp)
{
	return virt_to_phys(sp);
}

#ifdef CONFIG_SYSCTL
/*
 * ARGH!  ACPI people defined CTL_ACPI in linux/acpi.h rather than
 * linux/sysctl.h.
 *
 * This means our interface here won't survive long - it needs a new
 * interface.  Quick hack to get this working - use sysctl id 9999.
 */
#warning ACPI broke the kernel, this interface needs to be fixed up.
#define CTL_ACPI 9999
#define ACPI_S1_SLP_TYP 19

/*
 * Send us to sleep.
 */
static int sysctl_pm_do_suspend(void)
{
	int retval;

	retval = pm_send_all(PM_SUSPEND, (void *)3);

	if (retval == 0) {
		retval = pm_do_suspend();

		pm_send_all(PM_RESUME, (void *)0);
	}

	return retval;
}

static struct ctl_table pm_table[] =
{
	{ACPI_S1_SLP_TYP, "suspend", NULL, 0, 0600, NULL, (proc_handler *)&sysctl_pm_do_suspend},
	{0}
};

static struct ctl_table pm_dir_table[] =
{
	{CTL_ACPI, "pm", NULL, 0, 0555, pm_table},
	{0}
};

/*
 * Initialize power interface
 */
static int __init pm_init(void)
{
	register_sysctl_table(pm_dir_table, 1);
	return 0;
}

__initcall(pm_init);

#endif
